@@ -39,25 +39,10 @@ impl Client {
     ///
     /// # Examples
     ///
-    /// Get data
+    /// TODO: update once data types are crdt compliant
     ///
-    /// ```no_run
-    /// # extern crate tokio; use anyhow::Result;
-    /// # use safe_network::client::utils::test_utils::read_network_conn_info;
-    /// use safe_network::client::{Client, DEFAULT_QUERY_TIMEOUT};
-    /// use safe_network::types::ChunkAddress;
-    /// use xor_name::XorName;
-    /// # #[tokio::main] async fn main() { let _: Result<()> = futures::executor::block_on( async {
-    /// let head_chunk = ChunkAddress::Public(XorName::random());
-    /// # let bootstrap_contacts = Some(read_network_conn_info()?);
-    /// let mut client = Client::new(None, None, bootstrap_contacts, DEFAULT_QUERY_TIMEOUT).await?;
-    ///
-    /// // grab the random head of the blob from the network
-    /// let _data = client.read_blob(head_chunk, None, None).await?;
-    /// # Ok(())} );}
-    /// ```
     pub async fn read_blob(
-        &mut self,
+        &self,
         head_address: ChunkAddress,
         position: Option<usize>,
         len: Option<usize>,
@@ -91,25 +76,8 @@ impl Client {
     ///
     /// # Examples
     ///
-    /// Store data
+    /// TODO: update once data types are crdt compliant
     ///
-    /// ```no_run
-    /// # extern crate tokio; use anyhow::Result;
-    /// # use safe_network::client::utils::test_utils::read_network_conn_info;
-    /// use safe_network::client::{Client, DEFAULT_QUERY_TIMEOUT};
-    /// use safe_network::types::Token;
-    /// use std::str::FromStr;
-    /// # #[tokio::main] async fn main() { let _: Result<()> = futures::executor::block_on( async {
-    /// // Let's use an existing client, with a pre-existing balance to be used for write payments.
-    /// # let bootstrap_contacts = Some(read_network_conn_info()?);
-    /// let mut client = Client::new(None, None, bootstrap_contacts, DEFAULT_QUERY_TIMEOUT).await?;
-    /// # let initial_balance = Token::from_str("100")?; client.trigger_simulated_farming_payout(initial_balance).await?;
-    /// let data = b"some data".to_vec();
-    /// // grab the random head of the blob from the network
-    /// let _address = client.store_public_blob(&data).await?;
-    ///
-    /// # let balance_after_write = client.get_local_balance().await; assert_ne!(initial_balance, balance_after_write); Ok(()) } ); }
-    /// ```
     pub async fn store_public_blob(&self, data: &[u8]) -> Result<ChunkAddress, Error> {
         self.create_new_blob(data, true).await
     }
@@ -122,26 +90,8 @@ impl Client {
     ///
     /// # Examples
     ///
-    /// Store data
+    /// TODO: update once data types are crdt compliant
     ///
-    /// ```no_run
-    /// # extern crate tokio; use anyhow::Result;
-    /// # use safe_network::client::utils::test_utils::read_network_conn_info;
-    /// use safe_network::client::{Client, DEFAULT_QUERY_TIMEOUT};
-    /// use safe_network::types::Token;
-    /// use std::str::FromStr;
-    /// # #[tokio::main] async fn main() { let _: Result<()> = futures::executor::block_on( async {
-    /// // Let's use an existing client, with a pre-existing balance to be used for write payments.
-    /// # let bootstrap_contacts = Some(read_network_conn_info()?);
-    /// let mut client = Client::new(None, None, bootstrap_contacts, DEFAULT_QUERY_TIMEOUT).await?;
-    /// # let initial_balance = Token::from_str("100")?; client.trigger_simulated_farming_payout(initial_balance).await?;
-    /// let data = b"some data".to_vec();
-    /// // grab the random head of the blob from the network
-    /// let fetched_data = client.store_private_blob(&data).await?;
-    ///
-    /// println!( "{:?}", fetched_data ); // prints "some data"
-    /// # let balance_after_write = client.get_local_balance().await; assert_ne!(initial_balance, balance_after_write); Ok(()) } ); }
-    /// ```
     pub async fn store_private_blob(&self, data: &[u8]) -> Result<ChunkAddress, Error> {
         self.create_new_blob(data, false).await
     }
@@ -159,7 +109,7 @@ impl Client {
     }
 
     pub(crate) async fn fetch_blob_from_network(
-        &mut self,
+        &self,
         head_address: ChunkAddress,
         allow_cache: bool,
     ) -> Result<Chunk, Error> {
@@ -223,33 +173,9 @@ impl Client {
     ///
     /// # Examples
     ///
-    /// Remove data
+    /// TODO: update once data types are crdt compliant
     ///
-    /// ```no_run
-    /// # use safe_network::client::utils::test_utils::read_network_conn_info;
-    /// use safe_network::client::{Client, DEFAULT_QUERY_TIMEOUT};
-    /// use safe_network::types::Token;
-    /// use std::str::FromStr;
-    /// # #[tokio::main] async fn main() { let _: anyhow::Result<()> = futures::executor::block_on( async {
-    ///
-    /// // Let's use an existing client, with a pre-existing balance to be used for write payments.
-    /// # let bootstrap_contacts = Some(read_network_conn_info()?);
-    /// let mut client = Client::new(None, None, bootstrap_contacts, DEFAULT_QUERY_TIMEOUT).await?;
-    /// # let initial_balance = Token::from_str("100")?; client.trigger_simulated_farming_payout(initial_balance).await?;
-    /// let data = b"some private data".to_vec();
-    /// let address = client.store_private_blob(&data).await?;
-    ///
-    /// let _ = client.delete_blob(address).await?;
-    ///
-    /// // Now when we attempt to retrieve the blob, we should get an error
-    ///
-    /// match client.read_blob(address, None, None).await {
-    ///     Err(error) => eprintln!("Expected error getting blob {:?}", error),
-    ///     _ => return Err(anyhow::anyhow!("Should not have been able to retrieve this blob"))
-    /// };
-    /// #  Ok(())} );}
-    /// ```
-    pub async fn delete_blob(&mut self, head_chunk: ChunkAddress) -> Result<(), Error> {
+    pub async fn delete_blob(&self, head_chunk: ChunkAddress) -> Result<(), Error> {
         info!("Deleting blob at given address: {:?}", head_chunk);
 
         let mut chunk = self.fetch_blob_from_network(head_chunk, false).await?;
@@ -388,7 +314,7 @@ impl Client {
                 PrivateChunk::new(contents, self.public_key()).into()
             };
 
-            // If data map chunk is less thatn 1MB return it so it can be directly sent to the network
+            // If data map chunk is less that 1MB return it so it can be directly sent to the network
             if chunk.validate_size() {
                 return Ok(chunk);
             } else {
@@ -424,19 +350,71 @@ impl Client {
 mod tests {
     use super::{Chunk, ChunkAddress, Client, DataMap, DataMapLevel, Error};
     use crate::client::client_api::blob_storage::BlobStorage;
-    use crate::client::utils::{generate_random_vector, test_utils::create_test_client};
-    use crate::types::{PrivateChunk, PublicChunk, Token};
-    use crate::{retry_err_loop, retry_loop};
+    use crate::client::utils::{
+        generate_random_vector,
+        test_utils::{create_test_client, run_w_backoff},
+    };
+    use crate::retry_err_loop;
+    use crate::types::{PrivateChunk, PublicChunk};
     use anyhow::{anyhow, bail, Result};
     use bincode::deserialize;
+    use futures::future::join_all;
     use self_encryption::Storage;
-    use std::str::FromStr;
-    use tokio::time::Duration;
+    use tokio::time::{Duration, Instant};
+
+    // Test storing and getting public Blob.
+    #[tokio::test]
+    #[ignore = "too heavy for CI"]
+    pub async fn parallel_timings() -> Result<()> {
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+
+        let handles = (0..1000_usize)
+            .map(|i| (i, client.clone()))
+            .map(|(i, client)| {
+                tokio::spawn(async move {
+                    let value = generate_random_vector::<u8>(1000);
+                    let _ = client.store_public_blob(&value).await?;
+                    println!("Iter: {}", i);
+                    let res: Result<()> = Ok(());
+                    res
+                })
+            });
+
+        let results = join_all(handles).await;
+
+        for res1 in results {
+            if let Ok(res2) = res1 {
+                if res2.is_err() {
+                    println!("Error: {:?}", res2);
+                }
+            } else {
+                println!("Error: {:?}", res1);
+            }
+        }
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[ignore = "too heavy for CI"]
+    pub async fn one_by_one_timings() -> Result<()> {
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+
+        for i in 0..1000_usize {
+            let value = generate_random_vector::<u8>(1000);
+            let now = Instant::now();
+            let _ = client.store_public_blob(&value).await?;
+            let elapsed = now.elapsed();
+            println!("Iter: {}, in {} millis", i, elapsed.as_millis());
+        }
+
+        Ok(())
+    }
 
     // Test storing and getting public Blob.
     #[tokio::test]
     pub async fn public_blob_test() -> Result<()> {
-        let mut client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
         let value = generate_random_vector::<u8>(10);
         let (_, expected_address) = Client::blob_data_map(value.clone(), None).await?;
 
@@ -453,7 +431,8 @@ mod tests {
         assert_eq!(expected_address, public_address);
 
         // Assert that the blob was written
-        let fetched_data = retry_loop!(client.read_blob(public_address, None, None));
+        let fetched_data =
+            run_w_backoff(|| client.read_blob(public_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         // Test storing public chunk with the same value.
@@ -487,7 +466,7 @@ mod tests {
         assert_eq!(expected_address, priv_address);
 
         // Assert that the blob is stored.
-        let fetched_data = retry_loop!(client.read_blob(priv_address, None, None));
+        let fetched_data = run_w_backoff(|| client.read_blob(priv_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         // Test storing private chunk with the same value.
@@ -499,7 +478,8 @@ mod tests {
         let public_address = client.store_public_blob(&value).await?;
 
         // Assert that the public Blob is stored.
-        let fetched_data = retry_loop!(client.read_blob(public_address, None, None));
+        let fetched_data =
+            run_w_backoff(|| client.read_blob(public_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         // Delete Blob
@@ -528,7 +508,7 @@ mod tests {
         assert_eq!(new_addr, priv_address);
 
         // Assert that the Blob is stored again.
-        let fetched_data = retry_loop!(client.read_blob(priv_address, None, None));
+        let fetched_data = run_w_backoff(|| client.read_blob(priv_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         Ok(())
@@ -542,9 +522,10 @@ mod tests {
         let address = client.store_private_blob(&value).await?;
 
         // let's make sure we have all chunks stored on the network
-        let _ = retry_loop!(client.read_blob(address, None, None));
+        let _ = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
 
-        let fetched_data = retry_loop!(client.fetch_blob_from_network(address, false));
+        let fetched_data =
+            run_w_backoff(|| client.fetch_blob_from_network(address, false), 10).await?;
 
         let root_data_map = match deserialize(fetched_data.value())? {
             DataMapLevel::Root(data_map) => data_map,
@@ -575,27 +556,6 @@ mod tests {
         }
     }
 
-    #[tokio::test]
-    pub async fn blob_deletions_should_cost_put_price() -> Result<()> {
-        let mut client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
-
-        let address = client
-            .store_private_blob(&generate_random_vector::<u8>(10))
-            .await?;
-
-        let _ = retry_loop!(client.read_blob(address, None, None));
-
-        let balance_before_delete = client.get_balance().await?;
-        client.delete_blob(address).await?;
-        let new_balance = client.get_balance().await?;
-
-        // make sure we have _some_ balance
-        assert_ne!(balance_before_delete, Token::from_str("0")?);
-        assert_ne!(balance_before_delete, new_balance);
-
-        Ok(())
-    }
-
     // Test creating and retrieving a 1kb blob.
     #[tokio::test]
     pub async fn create_and_retrieve_1kb_pub_unencrypted() -> Result<()> {
@@ -619,11 +579,11 @@ mod tests {
         let size = 1024;
         let data = generate_random_vector(size);
 
-        let mut client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
         let address = client.store_public_blob(&data).await?;
 
         // let's make sure the public chunk is stored
-        let _ = retry_loop!(client.read_blob(address, None, None));
+        let _ = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
 
         // and now trying to read a private chunk with same address should fail
         let res = client
@@ -640,12 +600,12 @@ mod tests {
 
         let value = generate_random_vector(size);
 
-        let mut client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
 
         let address = client.store_private_blob(&value).await?;
 
         // let's make sure the private chunk is stored
-        let _ = retry_loop!(client.read_blob(address, None, None));
+        let _ = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
 
         // and now trying to read a public chunk with same address should fail
         let res = client
@@ -707,20 +667,25 @@ mod tests {
     async fn create_and_index_based_retrieve(size: usize) -> Result<()> {
         // Test read first half
         let data = generate_random_vector(size);
-        let mut client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
 
         let address = client.store_public_blob(&data).await?;
 
-        let fetched_data = retry_loop!(client.read_blob(address, None, Some(size / 2)));
+        let fetched_data =
+            run_w_backoff(|| client.read_blob(address, None, Some(size / 2)), 10).await?;
         assert_eq!(fetched_data, data[0..size / 2].to_vec());
 
         // Test read second half
         let data = generate_random_vector(size);
-        let mut client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
 
         let address = client.store_public_blob(&data).await?;
 
-        let fetched_data = retry_loop!(client.read_blob(address, Some(size / 2), Some(size / 2)));
+        let fetched_data = run_w_backoff(
+            || client.read_blob(address, Some(size / 2), Some(size / 2)),
+            10,
+        )
+        .await?;
         assert_eq!(fetched_data, data[size / 2..size].to_vec());
 
         Ok(())
@@ -730,7 +695,7 @@ mod tests {
     async fn gen_data_then_create_and_retrieve(size: usize, public: bool) -> Result<()> {
         let raw_data = generate_random_vector(size);
 
-        let mut client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
+        let client = create_test_client(Some(BLOB_TEST_QUERY_TIMEOUT)).await?;
 
         // generate address without storing to the network (public and unencrypted)
         let chunk = if public {
@@ -758,7 +723,7 @@ mod tests {
         };
 
         // now that it was put to the network we should be able to retrieve it
-        let fetched_data = retry_loop!(client.read_blob(address, None, None));
+        let fetched_data = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
         // then the content should be what we put
         assert_eq!(fetched_data, raw_data);
 
