@@ -14,17 +14,17 @@ mod map_storage;
 mod register_storage;
 mod sequence_storage;
 
+use crate::dbs::UsedSpace;
 use crate::messaging::{
     client::{
         ClientMsg, ClientSig, CmdError, DataCmd, DataExchange, DataQuery, ProcessMsg, QueryResponse,
     },
     Aggregation, DstLocation, EndUser, MessageId,
 };
-use crate::node::error::Error;
 use crate::node::{
     capacity::Capacity,
     node_ops::{MsgType, NodeDuties, NodeDuty, OutgoingMsg},
-    Result,
+    Error, Result,
 };
 use crate::routing::Prefix;
 use crate::types::{Chunk, PublicKey};
@@ -51,11 +51,11 @@ pub struct Metadata {
 }
 
 impl Metadata {
-    pub async fn new(path: &Path, max_capacity: u64, capacity: Capacity) -> Result<Self> {
+    pub async fn new(path: &Path, used_space: UsedSpace, capacity: Capacity) -> Result<Self> {
         let chunk_records = ChunkRecords::new(capacity);
-        let map_storage = MapStorage::new(path, max_capacity).await?;
-        let sequence_storage = SequenceStorage::new(path, max_capacity).await?;
-        let register_storage = RegisterStorage::new(path, max_capacity).await?;
+        let map_storage = MapStorage::new(path, used_space.max_capacity()); // to be removed so we don't care to implement this
+        let sequence_storage = SequenceStorage::new(path, used_space.max_capacity()); // to be removed so we don't care to implement this
+        let register_storage = RegisterStorage::new(path, used_space);
         let elder_stores = ElderStores::new(
             chunk_records,
             map_storage,
@@ -73,7 +73,7 @@ impl Metadata {
         origin: EndUser,
     ) -> Result<NodeDuty> {
         match self.elder_stores.read(query, id, requester, origin).await {
-            Err(Error::NoSuchChunk(_)) => Ok(NodeDuty::NoOp),
+            Err(Error::NoSuchData(_)) => Ok(NodeDuty::NoOp),
             res => res,
         }
     }
@@ -109,17 +109,17 @@ impl Metadata {
     }
 
     /// Adds a given node to the list of full nodes.
-    pub async fn increase_full_node_count(&mut self, node_id: PublicKey) {
+    pub async fn increase_full_node_count(&self, node_id: PublicKey) {
         self.elder_stores
-            .chunk_records_mut()
+            .chunk_records()
             .increase_full_node_count(node_id)
             .await
     }
 
     // When receiving the chunk from remaining holders, we ask new holders to store it.
-    pub async fn republish_chunk(&mut self, chunk: Chunk) -> Result<NodeDuty> {
+    pub async fn republish_chunk(&self, chunk: Chunk) -> Result<NodeDuty> {
         self.elder_stores
-            .chunk_records_mut()
+            .chunk_records()
             .republish_chunk(chunk)
             .await
     }
